(function(window, undefined) {

  if(window.iw === undefined) window.iw = {};
    
  iw.testsuites = new Vue({
    el: '#testsuites-app',
    data : {
    
      info : {},
      
      assertionTypes: {},
      
      suites: {},
      
      filteredSuites : {},
      
      testSuiteData: {
        baseDir: "",
        packageName: "",
        service: "",
        testSuite: {}
      },

      showrundiv           : false,
      clearTestResults     : false,
      runIndices           : [],
      remoteTestSuiteAlias : "",
      
      showexecutediv: false,
      
      packageFilter   : "",
      interfaceFilter : "",
      serviceFilter   : "",
      executeType     : "",
      remoteAlias     : "",
      running         : false,
      
      showdeletediv: false,
      deleteIndices: [],
      removeDataOnDisk: true,
      
      showpipelinediv: false,
      pipeline: {
        data: {},
        isproblematic: false,
        path: "",
        raw: ""
      },
      dirtyCallbackFiles: [],
      
      //Contains a map of current
      //and original callback file names
      //{current: "", original: ""}
      dirtyCallbackRenames: {},
      
      isdirtypipeline : false,
      
      showeditdiv: false,
      
      showraweditor: false,
      
      showselectservicedialog : false,
      
      selectedService: "",
      
      localPackages : [],
      
      message: "",
      
      success: true,
      
      aliases: [],
      
      config : {}
       
    },
    computed : {
    
      executeUrl : function() {
        let url = "";
        if(this.info.host) {
          url = this.info.host.protocol + "://" + this.info.host.server + ":" + this.info.host.port + "/invoke/iw.test.execution.pub/run";
          
          let query = "";
        
          if(this.executeType !== "") {
            if(query.length > 0) query += "&";
            query += "type=" + this.executeType;
          }
        
          if(this.packageFilter !== "") {
            if(query.length > 0) query += "&";
            query += "packageFilter=" + encodeURIComponent(this.packageFilter);
          }
        
          if(this.interfaceFilter !== "") {
            if(query.length > 0) query += "&";
            query += "interfaceFilter=" + encodeURIComponent(this.interfaceFilter);
          }
        
          if(this.serviceFilter !== "") {
            if(query.length > 0) query += "&";
            query += "serviceFilter=" + encodeURIComponent(this.serviceFilter);
          }
        
          if(this.remoteAlias !== "") {
            if(query.length > 0) query += "&";
            query += "remote-alias=" + encodeURIComponent(this.remoteAlias);          
          }
          if(query !== "") url += "?" + query;
          
          this.getFilteredTestSuites()
          
        }
        iw.log("Computed url: " + url, 3);
        return url
      }
    },
    methods : {
    
      //Use this function for showing the result of actions, not for data retrieval
      response :function(data) {
         iw.log("Logging response from IS", 4);
         iw.log(JSON.stringify(data), 4);
         iw.testsuites.showMessage(data["$message"], data["$success"] === "true" , 2500);        
         //iw.testsuites.getAllTestSuites();
      },
      
      error : function(msg) { 
        iw.log(msg, 0);
        iw.testsuites.showMessage(msg, false , 2500);       
      },
      
      showMessage : function(msg, success, timeout) {  
         iw.testsuites.message = msg;
         iw.testsuites.success = success;
         if(typeof(timeout) === "number") {
            setTimeout(function(){iw.testsuites.message=""}, timeout);

         }
      },
      
      hideMessage : function() {
         iw.testsuites.message = "";
      },
      
      getInfo : function() {
        iw.invoke("iw.test.admin:info", {}, function(data) {
          iw.testsuites.info = data;          
        });
      },
      
      getConfig : function() {
        iw.invoke("iw.test.config:get", {}, function(data) {
          iw.testsuites.config = data;          
        });        
      },
      
      getAssertionTypes : function() {
        iw.invoke("iw.test.assertion.service:getAssertionTypes", {}, function(data) {
          iw.testsuites.assertionTypes = data['assertion-types'];          
        })      
      },
      
      getAllTestSuites : function() {
        iw.invoke("iw.test.ui.testsuite:list", {}, function(data) {
          iw.testsuites.suites = data;          
          iw.log("Test suite data retrieved from server", 3);
          }       
        )
        iw.invoke("iw.test.alias:list", {}, function(data) {
          iw.testsuites.aliases = data.aliases;          
          iw.log("Aliases retrieved from server", 3);
          }
        )
      },
      
      getFilteredTestSuites : function() {
        iw.invoke("iw.test.ui.testsuite:list",
          {"packageFilter": this.packageFilter, "interfaceFilter" : this.interfaceFilter, "serviceFilter": this.serviceFilter, "type": this.executeType}, 
            function(data) {
              iw.testsuites.filteredSuites = data;          
              iw.log("Filtered est suites retrieved from server", 3);
            }       
        )
      },      
      
      getTestSuite : function(testSuiteServiceName) {
        iw.testsuites.testSuiteData.service      = testSuiteServiceName;

        iw.invoke(testSuiteServiceName, {"definition-only" : true}, function(data) {
          iw.testsuites.testSuiteData.testSuite    = data["test-suite"];
          iw.testsuites.testSuiteData.baseDir      = data["base-dir"];
        }        
        
      )},
      
      showRunDialog : function(testSuiteServiceName, packageName) {
        iw.log("Show run dialog called for service: " + testSuiteServiceName, 3)
        iw.testsuites.testSuiteData.packageName = packageName
        iw.testsuites.getTestSuite(testSuiteServiceName);
        iw.testsuites.showrundiv = true;
      },
      
      hideRunDialog : function() {
        iw.testsuites.showrundiv = false;        
      },
      
      runTestSuite : function(service, clearTestResults, runIndices, remoteTestSuiteAlias) {
        iw.log("Running test suite '" + service +"'" + runIndices + "; remote alias: " + remoteTestSuiteAlias, 3)
        var message = "";
        if(clearTestResults) {
          iw.invoke("iw.test.results.pub:clear", {}, function(){})
          iw.testsuites.clearTestResults = false;
          message = "Results cleared\n";
        }
        var params = {"service": service, "async" : true, "pipeline" : {"index" : runIndices.map(i => {return ++i}).join(","), "remote-alias" : remoteTestSuiteAlias }}
        iw.invoke("iw.test.execution.util:invoke", params, function(){
        
        });
        message += service + " submitted for execution";
        iw.testsuites.showMessage(message, true, 2500);
        iw.testsuites.showrundiv = false;
      },
      
      showExecuteDialog : function() {
        iw.testsuites.showexecutediv = true;  
      },
      
      hideExecuteDialog : function() {
        iw.testsuites.showexecutediv = false;      
      },
      
      executeTestSuites : function() {
        iw.log("Executing test suites. type: " + this.executeType + "; package filter: " + this.packageFilter + "; interface filter: " + this.interfaceFilter + "; service filter: " + this.serviceFilter + "; remote alias: " + this.remoteAlias, 3);
        var pipeline = {
            "type"            : this.executeType,
            "packageFilter"   : this.packageFilter,
            "interfaceFilter" : this.interfaceFilter,
            "serviceFilter"   : this.serviceFilter,
            "remote-alias"    : this.remoteAlias
        };
        var params = {
          "service" : "iw.test.execution.pub:run",
          "async"   : "true",
          "pipeline" : pipeline
        }
        //iw.invoke("iw.test.execution.util:invoke", params, function(){});
        iw.invoke("iw.test.execution.pub:run", pipeline, function(data){
          iw.testsuites.running = false;
          iw.testsuites.showMessage(data["$message"], data["$success"] === "true" , 5000);
        });
        
        iw.testsuites.message = "Running test suites..";
        iw.testsuites.running = true;
        iw.testsuites.showexecutediv = false;
      },
      
      showDeleteDialog : function(testSuiteServiceName, packageName) {
        iw.testsuites.testSuiteData.packageName = packageName
        iw.testsuites.testSuiteData.service     = testSuiteServiceName;
        iw.testsuites.getTestSuite(testSuiteServiceName);
        iw.testsuites.showdeletediv = true;      
      },
      
      hideDeleteDialog : function() {
        iw.testsuites.showdeletediv = false;
      },
      
      deleteTestSuite: function(testSuiteServiceName, deleteIndices) {
        var params = {"test-suite-service"  : testSuiteServiceName, 
                      "delete-data-on-disk" : iw.testsuites.removeDataOnDisk,
                      "index" : deleteIndices.join(",")}
                        
        iw.invoke("iw.test.ui.testsuite:remove", params, iw.testsuites.response, iw.testsuites.error);       
        iw.testsuites.showdeletediv = false;
        iw.testsuites.deleteIndices = [];
      },
      
      showEditDialog : function(testSuiteServiceName, packageName) {
        iw.log("Show edit dialog called", 3)
        iw.testsuites.testSuiteData.packageName = packageName
        iw.testsuites.getTestSuite(testSuiteServiceName);
        iw.testsuites.showeditdiv = true;
      },
      
      hideEditDialog : function() {
        iw.testsuites.showeditdiv = false;      
        //TODO: Call service to remove files superfluous callback files.
        iw.testsuites.dirtyCallbackFiles.map(function(f) {
        
          iw.log("Deleting file " + f + "; base-dir: " + iw.testsuites.testSuiteData.baseDir, 3);
          var params = {"file-name": f, "base-dir": iw.testsuites.testSuiteData.baseDir};
          iw.invoke("iw.test.util.file:remove", params, function(){}, iw.testsuites.error );        
        
        });
        for(var p in iw.testsuites.dirtyCallbackRenames) {
          if(iw.testsuites.dirtyCallbackRenames.hasOwnProperty(p)) {
            //Rename files back
            iw.log("Renaming dirty callback files back to original name. current name: " + p +  "; original name: " + iw.testsuites.dirtyCallbackRenames[p]);
            var params = {"existing-file-name": p, "new-file-name": iw.testsuites.dirtyCallbackRenames[p], "base-dir": iw.testsuites.testSuiteData.baseDir};
            iw.invoke("iw.test.util.file:rename", params, function(){}, iw.testsuites.error );
            
          }
        }
        iw.testsuites.dirtyCallbackFiles = []
        iw.testsuites.dirtyCallbackRenames = {}
        iw.testsuites.hidePipelineDialog();
      },
      
      saveTestSuite : function() {
        iw.log("About to save test suite", 3);
        //TODO: Call service to remove files superfluous callback files.
        var params   = {"test-suite-service": iw.testsuites.testSuiteData.service, 
                        "test-suite"        : iw.testsuites.testSuiteData.testSuite,
                        "base-dir"          : iw.testsuites.testSuiteData.baseDir}
       
       //Prepare data. 'lax' must be converted to a string; individual stub files should be removed        
       for(var i = 0 ; i < iw.testsuites.testSuiteData.testSuite["test-cases"].length; i++) {
          //First check 'lax'. The checkbox has made it a boolean
           var expected = iw.testsuites.testSuiteData.testSuite["test-cases"][i].expected;
           if(expected !== undefined && expected.lax === true) {
             expected.lax = "true"
           }
           //The descend into the dynamic test case stubs
           //When retrieving the test suite definit
           if (iw.testsuites.testSuiteData.testSuite["test-cases"][i].stubs) {
            iw.log("test-case " + [i] + " has stubs ", 4)
            for(var j = 0 ; j < iw.testsuites.testSuiteData.testSuite["test-cases"][i].stubs.length; j++) {
              var stub = iw.testsuites.testSuiteData.testSuite["test-cases"][i].stubs[j]

              if(stub.dynamic) {                
                iw.log("stub["+j+"] is dynamic", 4)
                for(var p in stub.dynamic) {
                    iw.log("property "+p+" is of type: " + typeof(stub.dynamic[p]), 4)
                    if( typeof(stub.dynamic[p]) === "object") {
                      delete stub.dynamic[p]
                      iw.log("deleted property " + p, 4);  
                    } 
                }
              } 
            }
           }            
        }
        
        iw.invoke("iw.test.ui.testsuite:save", params, iw.testsuites.response, iw.testsuites.error);
               
        iw.testsuites.dirtyCallbackFiles = [];
        iw.testsuites.dirtyCallbackRenames = {};
        iw.testsuites.hideEditDialog();
      },
      
      showPipelineDialog : function(baseDir, path) {
        iw.testsuites.getPipeline(baseDir, path)
        iw.testsuites.showpipelinediv = true;
      },
      
      hidePipelineDialog : function() {
        iw.testsuites.showpipelinediv = false;
      },
      
      showJSONEditor : function() {
        iw.testsuites.showraweditor = false;
      },
      
      showXMLEditor : function() {
        iw.testsuites.showraweditor = true;
      },
      
      getPipeline : function(baseDir, path) {
        iw.testsuites.pipeline.path = path;
        var params = {"$path": path, "$baseDir": baseDir, "$include-raw": "true"}
        iw.invoke("iw.test.util.pub:restorePipelineFromFile", params, function(data) {
          //iw.testsuites.pipeline.data = JSON.stringify(data['$document'])
          iw.testsuites.pipeline.isproblematic = ("true" === data['$problematic']);


          //Raw data 
          iw.testsuites.pipeline.raw = data['$raw'];
          iw.xmleditor.getSession().setValue(iw.testsuites.pipeline.raw, 1);

          //JSON
          iw.peditor.set(data['$document'])
          iw.peditor.setName("pipeline");
          iw.peditor.expandAll();
          
          //Reset isdirtypipeline, so the save button becomes disabled
          iw.testsuites.isdirtypipeline = false;

          //Explicitly show the XML editor, else the text won't repaint        
          iw.testsuites.showXMLEditor();
          //iw.testsuites.showraweditor = iw.testsuites.pipeline.isproblematic;

        }, iw.testsuites.error)
      },
      
      savePipeline : function() {
        var params = {"$path"    : iw.testsuites.pipeline.path, 
                      "$baseDir" : iw.testsuites.testSuiteData.baseDir,
                      "$package" : iw.testsuites.testSuiteData.packageName}
                      
        if(iw.testsuites.showraweditor) {
          params["$raw"] = iw.xmleditor.getValue();
        } else {
          params["$document"] = iw.peditor.get();
        }
        
        iw.invoke("iw.test.util.pub:savePipelineToFile", params, iw.testsuites.response, iw.testsuites.error)
        iw.testsuites.hidePipelineDialog();
        
      },
      
      addAssertion: function(assertions) {
        assertions.push({"path": "[path]", "check": "equals", "value":"[value]"});
      },
      
      removeAssertion: function(assertions, index) {
        assertions.splice(index, 1);
      },
      
      addCallback : function(testCase) {
        iw.log("Add callback called", 3);
        if(testCase.callbacks === undefined) {
          Vue.set(testCase, "callbacks", []);
        }
        
        testCase.callbacks.push({service: "", event: "start", expected: {"file-name": "", lax: "true" }});
      },
      
      removeCallback : function(callbacks, index) {
        iw.log("Deleting callback["+index+"]; file name: " + callbacks[index]["file-name"] + "; base-dir: " + iw.testsuites.testSuiteData.baseDir, 3);
        iw.log(callbacks[index], 3);
        if(callbacks[index].expected["file-name"] !== "") {
          iw.log("Deleting file " + callbacks[index].expected["file-name"] + "; base-dir: " + iw.testsuites.testSuiteData.baseDir, 3);
          var params = {"file-name": callbacks[index].expected["file-name"], "base-dir": iw.testsuites.testSuiteData.baseDir};
          iw.invoke("iw.test.util.file:remove", params, function(){}, iw.testsuites.error );        
        }
        
        callbacks.splice(index, 1);
      },
      setCallbackService($event, cb) {
        iw.log("Set callback service called. Service: " +$event,3);
      },
      
      showSelectCallbackServiceInput : function(testCase, callback) {
        iw.log("showSelectServiceInput called", 3);
        iw.testsuites.showselectservicedialog = true;
        iw.testsuites.currentTestCase = testCase;
        iw.testsuites.currentCallback = callback;
      },
      
      hideSelectServiceInput : function() {
        iw.testsuites.showselectservicedialog = false;      
        iw.testsuites.currentTestCase = {};
        iw.testsuites.currentCallback = {};        
      },
      
      assignSelectedService : function(service) {
        iw.log("Assign Selected Service called. Service: " + service, 3);
        iw.testsuites.showselectservicedialog = false;
        if (iw.testsuites.currentCallback.service !== service) {
          iw.testsuites.handleCallbackServiceChange(iw.testsuites.currentTestCase, iw.testsuites.currentCallback, 'service', service)
        }
      },
      
      handleCallbackServiceChange : function(tc, cb, field, value) {
        iw.log("Callback changed. Testcase id: " + tc.id + ";dir: " + tc.dir, 3);
        if(field === 'service') {
          iw.log("Old service: " + cb.service +"; new service: " + value, 3);
          cb.service = value;
        }
        if(field === 'event') {
          iw.log("Old event: " + cb.event +"; new event: " + value);
          cb.event = value;
        }
        if(cb.service !== "") {
          var cbFileName = iw.testsuites.composeCallbackFileName(tc.dir, cb.service, cb.event);
          if(cb.expected["file-name"] === "") {
            //create empty pipeline
            iw.log("Callback pipeline created. Name: " + cbFileName, 3);
            cb.expected["file-name"] = cbFileName;
            var params = {"$document": {}, "$path" : cb.expected["file-name"], "$baseDir": iw.testsuites.testSuiteData.baseDir}
            iw.invoke("iw.test.util.pub:savePipelineToFile", params, function(){}, iw.testsuites.error   );

            iw.testsuites.dirtyCallbackFiles.push(cbFileName);

          } else {
            //rename existing pipeline
            iw.log("Callback pipeline file renamed. old name: " + cb.expected["file-name"] +  "; new name: " + cbFileName);
            var params = {"existing-file-name": cb.expected["file-name"], "new-file-name": cbFileName, "base-dir": iw.testsuites.testSuiteData.baseDir};
            iw.invoke("iw.test.util.file:rename", params, function(){}, iw.testsuites.error );
                        
            //Keep track of the renames. Revert in case of a cancel
            if(iw.testsuites.dirtyCallbackRenames[cb.expected["file-name"]]) {
               iw.testsuites.dirtyCallbackRenames[cbFileName] = iw.testsuites.dirtyCallbackRenames[cb.expected["file-name"]];
               delete iw.testsuites.dirtyCallbackRenames[cb.expected["file-name"]]
            } else {
              iw.testsuites.dirtyCallbackRenames[cbFileName] = cb.expected["file-name"]
            }
            cb.expected["file-name"] = cbFileName;            
          }
        } else if(cb.expected["file-name"] !== "") {
            //remove existing pipeline
            iw.log("Callback pipeline removed");
            var params = {"file-name": cb.expected["file-name"], "base-dir": iw.testsuites.testSuiteData.baseDir};
            iw.invoke("iw.test.util.file:remove", params, function(){}, iw.testsuites.error );         
            iw.testsuites.dirtyCallbackFiles.splice(iw.testsuites.dirtyCallbackFiles.indexOf(cb.expected["file-name"]),1);
            cb.expected["file-name"] = "";
        }
      },
      
      composeCallbackFileName : function (testCaseDir, service, event) {
        return testCaseDir + "/callbacks/" +  service.replace(":",".") + "-" +event + ".xml"
      },

      listLocalPackages : function(alias) {
		   var params = {};
		   iw.invoke("iw.test.ui.ns:listPackages", params, function(data) {
		      iw.log("Retrieved local packages", 3);

            iw.testsuites.localPackages = data.packages.filter(function(a){                 
                 var result = a["system-package"] !== "true" && a.enabled === "true";                 
                 return result;
            }).sort(function(a,b){ if(a.name >= b.name) return 1; else return -1});
            
                
         }, iw.testsuites.error);
		  }
      
    }
   
  });  
  
  iw.testsuites.getInfo();
  iw.testsuites.getConfig();
  iw.testsuites.getAssertionTypes();
  iw.testsuites.getAllTestSuites(); 
  iw.testsuites.listLocalPackages();
  var container = document.getElementById("pipelineeditor");
  var options = { modes:["tree","code","text"],
                  enableTransform: false,
                  onChange: function() {iw.testsuites.isdirtypipeline = true},
                  onClassName: function(c) {
                    //iw.log("on-class-name: " + c.path + " " + c.field + " " + c.value, 3)
                  }
                }

   iw.peditor = new JSONEditor(container, options);

   iw.xmleditor = JSONEditor.ace.edit("xmleditor");
   iw.xmleditor.on("change", function() {iw.log("XML changed",3); iw.testsuites.isdirtypipeline = true});
   /* Loading XML support
   window.ace = JSONEditor.ace;
   iw.xmleditor.getSession().setMode("ace/mode/xml");
   */
})(this, undefined);